<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace MapModule\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\Datasource\RepositoryInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use MapModule\Model\Entity\Mapsummaryitem;

/**
 * Mapsummaryitems Model
 *
 * @property MapsTable&BelongsTo $Maps
 *
 * @method Mapsummaryitem get($primaryKey, $options = [])
 * @method Mapsummaryitem newEntity($data = null, array $options = [])
 * @method Mapsummaryitem[] newEntities(array $data, array $options = [])
 * @method Mapsummaryitem|false save(EntityInterface $entity, $options = [])
 * @method Mapsummaryitem saveOrFail(EntityInterface $entity, $options = [])
 * @method Mapsummaryitem patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method Mapsummaryitem[] patchEntities($entities, array $data, array $options = [])
 * @method Mapsummaryitem findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin TimestampBehavior
 */
class MapsummaryitemsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('mapsummaryitems');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('Maps', [
            'foreignKey' => 'map_id',
            'joinType'   => 'INNER',
            'className'  => 'MapModule.Maps',
        ]);

        $this->hasMany('Hosts');
        $this->hasMany('Hostgroups');
        $this->hasMany('Services');
        $this->hasMany('Servicegroups');
    }

    public function bindCoreAssociations(RepositoryInterface $coreTable) {
        switch ($coreTable->getAlias()) {
            case 'Hosts':
                $coreTable->hasMany('Mapsummaryitems', [
                    'className'  => 'MapModule.Mapsummaryitems',
                    'dependent'  => true,
                    'foreignKey' => 'object_id',
                    'joinType'   => 'INNER',
                    'conditions' => [
                        'type' => 'host'
                    ]
                ]);
                break;
            case 'Hostgroups':
                $coreTable->hasMany('Mapsummaryitems', [
                    'className'  => 'MapModule.Mapsummaryitems',
                    'dependent'  => true,
                    'foreignKey' => 'object_id',
                    'joinType'   => 'INNER',
                    'conditions' => [
                        'type' => 'hostgroup'
                    ]
                ]);
                break;
            case 'Services':
                $coreTable->hasMany('Mapsummaryitems', [
                    'className'  => 'MapModule.Mapsummaryitems',
                    'dependent'  => true,
                    'foreignKey' => 'object_id',
                    'joinType'   => 'INNER',
                    'conditions' => [
                        'type' => 'service'
                    ]
                ]);
                break;
            case 'Servicegroups':
                $coreTable->hasMany('Mapsummaryitems', [
                    'className'  => 'MapModule.Mapsummaryitems',
                    'dependent'  => true,
                    'foreignKey' => 'object_id',
                    'joinType'   => 'INNER',
                    'conditions' => [
                        'type' => 'servicegroup'
                    ]
                ]);
                break;
        }
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn(['map_id'], 'Maps'));

        return $rules;
    }

    /**
     * @param $id
     * @return bool
     */
    public function existsById($id) {
        return $this->exists(['Mapsummaryitems.id' => $id]);
    }

    /**
     * @param $objectId
     * @param $mapId
     * @return array
     */
    public function getMapsummaryitemsForMaps($objectId, $mapId) {
        $query = $this->find()
            ->select(['Mapsummaryitems.object_id'])
            ->where([
                'Mapsummaryitems.object_id' => $objectId,
                'Mapsummaryitems.map_id'    => $mapId,
                'Mapsummaryitems.type'      => 'map',
            ]);

        $result = $query->first();
        if (empty($result)) {
            return [];
        }
        return $query->toArray();
    }

    /**
     * @param $mapId
     * @return array
     */
    public function allVisibleMapsummaryitems($mapId) {
        $query = $this->find()
            ->join([
                [
                    'table'      => 'maps',
                    'type'       => 'INNER',
                    'alias'      => 'Maps',
                    'conditions' => 'Maps.id = Mapsummaryitems.map_id',
                ]
            ])
            ->select([
                'Mapsummaryitems.map_id',
                'Mapsummaryitems.object_id',
            ])
            ->where([
                'Mapsummaryitems.type'          => 'map',
                'Mapsummaryitems.map_id != '    => $mapId,
                'Mapsummaryitems.object_id != ' => $mapId
            ]);

        $query->disableHydration();
        $result = $query->all();
        if (empty($result)) {
            return [];
        }
        return $result->toArray();
    }
}
