<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Authenticator;

use Authentication\Authenticator\AbstractAuthenticator;
use Authentication\Authenticator\AuthenticationRequiredException;
use Authentication\Authenticator\Result;
use Authentication\Authenticator\ResultInterface;
use Authentication\Authenticator\StatelessInterface;
use Psr\Http\Message\ServerRequestInterface;

class ApikeyAuthenticator extends AbstractAuthenticator implements StatelessInterface {

    /**
     * Authenticate a user based on the request information.
     *
     * @param ServerRequestInterface $request Request to get authentication information from.
     * @return ResultInterface Returns a result object.
     */
    public function authenticate(ServerRequestInterface $request): ResultInterface {
        $data = ['apikey' => null];

        if (!empty($request->getHeaderLine('Authorization'))) {
            $authHeaderKey = $this->getConfig('apikeyPrefix');
            $data['apikey'] = trim(substr($request->getHeaderLine($this->getConfig('header')), strlen($authHeaderKey)));
        }
        if (isset($request->getQueryParams()[$this->getConfig('queryParam')])) {
            $data['apikey'] = trim($request->getQueryParams()[$this->getConfig('queryParam')]);
        }

        $user = $this->_identifier->identify($data);
        if (empty($user) || $user === null) {
            return new Result(null, Result::FAILURE_IDENTITY_NOT_FOUND, $this->_identifier->getErrors());
        }
        //$request->withoutHeader('Set-Cookie');

        return new Result($user, Result::SUCCESS);
    }


    /**
     * Create a challenge exception
     *
     * Create an exception with the appropriate headers and response body
     * to challenge a request that has missing or invalid credentials.
     *
     * This is primarily used by authentication methods that use the WWW-Authorization
     * header.
     *
     * @param ServerRequestInterface $request The request.
     * @return void
     * @throws AuthenticationRequiredException
     */
    public function unauthorizedChallenge(ServerRequestInterface $request): void {
        //throw new AuthenticationRequiredException([], '');
    }
}
