<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;


use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\AcknowledgedHostConditions;
use itnovum\openITCOCKPIT\Core\AcknowledgedServiceConditions;
use itnovum\openITCOCKPIT\Core\AngularJS\Request\AcknowledgementsControllerRequest;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\AcknowledgementService;
use itnovum\openITCOCKPIT\Core\Views\BBCodeParser;
use itnovum\openITCOCKPIT\Database\PaginateOMat;


/**
 * Class AcknowledgementsController
 * @package App\Controller
 */
class AcknowledgementsController extends AppController {

    public function host() {
        if (!$this->isAngularJsRequest()) {
            //Only ship html template
            return;
        }

        $session = $this->request->getSession();
        $session->close();
        $AngularAcknowledgementsControllerRequest = new AcknowledgementsControllerRequest($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $AngularAcknowledgementsControllerRequest->getPage());

        //Process conditions
        /** @var AcknowledgedHostConditions $Conditions */
        $Conditions = new AcknowledgedHostConditions();
        $Conditions->setFrom($AngularAcknowledgementsControllerRequest->getFrom());
        $Conditions->setTo($AngularAcknowledgementsControllerRequest->getTo());
        $Conditions->setStates($AngularAcknowledgementsControllerRequest->getHostStates());
        $Conditions->setOrder($AngularAcknowledgementsControllerRequest->getOrderForPaginator('AcknowledgementHosts.entry_time', 'desc'));
        $Conditions->setConditions($AngularAcknowledgementsControllerRequest->getHostFilters());

        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();

        $AcknowledgementHostsTable = TableRegistry::getTableLocator()->get('AcknowledgementHosts');

        //Query acknowledgements records
        $BBCodeParser = new BBCodeParser();
        $all_acknowledgements = [];

        foreach ($AcknowledgementHostsTable->getAcknowledgements($Conditions, $PaginateOMat) as $AcknowledgementHost) {
            /** @var AcknowledgementHost $acknowledgement */
            $Acknowledgement = new \itnovum\openITCOCKPIT\Core\Views\AcknowledgementHost($AcknowledgementHost->toArray(), $UserTime);

            $acknowledgementArray = $Acknowledgement->toArray();
            $acknowledgementArray['comment_data'] = $BBCodeParser->nagiosNl2br($BBCodeParser->asHtml($acknowledgementArray['comment_data'], true));

            $all_acknowledgements[] = [
                'AcknowledgedHost' => $acknowledgementArray
            ];
        }

        $this->set('all_acknowledgements', $all_acknowledgements);
        $toJson = ['all_acknowledgements', 'paging'];
        if ($this->isScrollRequest()) {
            $toJson = ['all_acknowledgements', 'scroll'];
        }
        $this->viewBuilder()->setOption('serialize', $toJson);
    }

    public function service() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        $session = $this->request->getSession();
        $session->close();

        $AngularAcknowledgementsControllerRequest = new AcknowledgementsControllerRequest($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $AngularAcknowledgementsControllerRequest->getPage());

        //Process conditions
        $Conditions = new AcknowledgedServiceConditions();
        $Conditions->setFrom($AngularAcknowledgementsControllerRequest->getFrom());
        $Conditions->setTo($AngularAcknowledgementsControllerRequest->getTo());
        $Conditions->setStates($AngularAcknowledgementsControllerRequest->getServiceStates());
        $Conditions->setOrder($AngularAcknowledgementsControllerRequest->getOrderForPaginator('AcknowledgementServices.entry_time', 'desc'));
        $Conditions->setConditions($AngularAcknowledgementsControllerRequest->getServiceFilters());

        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();

        $AcknowledgementServicesTable = TableRegistry::getTableLocator()->get('AcknowledgementServices');

        //Query acknowledgements records
        $BBCodeParser = new BBCodeParser();
        $all_acknowledgements = [];
        foreach ($AcknowledgementServicesTable->getAcknowledgements($Conditions, $PaginateOMat) as $acknowledgement) {
            $AcknowledgedService = new AcknowledgementService($acknowledgement, $UserTime);
            $acknowledgementArray = $AcknowledgedService->toArray();
            $acknowledgementArray['comment_data'] = $BBCodeParser->nagiosNl2br($BBCodeParser->asHtml($acknowledgementArray['comment_data'], true));
            $all_acknowledgements[] = [
                'AcknowledgedService' => $acknowledgementArray
            ];
        }

        $this->set('all_acknowledgements', $all_acknowledgements);
        $toJson = ['all_acknowledgements', 'paging'];
        if ($this->isScrollRequest()) {
            $toJson = ['all_acknowledgements', 'scroll'];
        }
        $this->viewBuilder()->setOption('serialize', $toJson);
    }
}
