<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;

use App\Model\Entity\DowntimeHost;
use App\Model\Entity\DowntimeService;
use Cake\Datasource\Exception\RecordNotFoundException;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\AngularJS\Request\HostDowntimesControllerRequest;
use itnovum\openITCOCKPIT\Core\AngularJS\Request\ServiceDowntimesControllerRequest;
use itnovum\openITCOCKPIT\Core\DowntimeHostConditions;
use itnovum\openITCOCKPIT\Core\DowntimeServiceConditions;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\Downtime;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * Class DowntimesController
 * @package App\Controller
 */
class DowntimesController extends AppController {

    public function host() {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML template
            return;
        }

        $HostDowntimesControllerRequest = new HostDowntimesControllerRequest($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $HostDowntimesControllerRequest->getPage());

        //Process conditions
        $DowntimeHostConditions = new DowntimeHostConditions();
        $DowntimeHostConditions->setFrom($HostDowntimesControllerRequest->getFrom());
        $DowntimeHostConditions->setTo($HostDowntimesControllerRequest->getTo());
        $DowntimeHostConditions->setHideExpired($HostDowntimesControllerRequest->hideExpired());
        $DowntimeHostConditions->setIsRunning($HostDowntimesControllerRequest->isRunning());
        $DowntimeHostConditions->setOrder($HostDowntimesControllerRequest->getOrderForPaginator('DowntimeHosts.scheduled_start_time', 'desc'));
        $DowntimeHostConditions->setConditions($HostDowntimesControllerRequest->getIndexFilters());

        $DowntimehistoryHostsTable = TableRegistry::getTableLocator()->get('DowntimeHosts');

        $hostDowntimes = $DowntimehistoryHostsTable->getDowntimes($DowntimeHostConditions, $PaginateOMat);


        //Prepare data for API
        $UserTime = UserTime::fromUser($this->getUser());

        foreach ($hostDowntimes as $hostDowntime) {
            /** @var DowntimeHost $hostDowntime */
            $hostDowntime->setUserTime($UserTime);
        }

        $this->set('all_host_downtimes', $hostDowntimes);
        $this->viewBuilder()->setOption('serialize', ['all_host_downtimes']);
    }

    public function service() {
        if (!$this->isAngularJsRequest()) {
            return;
        }

        $ServiceDowntimesControllerRequest = new ServiceDowntimesControllerRequest($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $ServiceDowntimesControllerRequest->getPage());

        //Process conditions
        $DowntimeServiceConditions = new DowntimeServiceConditions();
        $DowntimeServiceConditions->setFrom($ServiceDowntimesControllerRequest->getFrom());
        $DowntimeServiceConditions->setTo($ServiceDowntimesControllerRequest->getTo());
        $DowntimeServiceConditions->setHideExpired($ServiceDowntimesControllerRequest->hideExpired());
        $DowntimeServiceConditions->setIsRunning($ServiceDowntimesControllerRequest->isRunning());
        $DowntimeServiceConditions->setOrder($ServiceDowntimesControllerRequest->getOrderForPaginator('DowntimeServices.scheduled_start_time', 'desc'));
        $DowntimeServiceConditions->setConditions($ServiceDowntimesControllerRequest->getIndexFilters());


        $DowntimehistoryServicesTable = TableRegistry::getTableLocator()->get('DowntimeServices');


        $serviceDowntimes = $DowntimehistoryServicesTable->getDowntimes($DowntimeServiceConditions, $PaginateOMat);

        //Prepare data for API
        $all_service_downtimes = [];
        $UserTime = UserTime::fromUser($this->getUser());

        foreach ($serviceDowntimes as $serviceDowntime) {
            /** @var DowntimeService $serviceDowntime */
            $serviceDowntime->setUserTime($UserTime);
        }

        $this->set('all_service_downtimes', $serviceDowntimes);
        $this->viewBuilder()->setOption('serialize', ['all_service_downtimes']);
    }

    public function validateDowntimeInputFromAngular() {
        if (!$this->isAngularJsRequest() || !$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }

        $error = ['Downtime' => []];
        $data = $this->request->getData();
        if (!isset($data['comment']) || strlen($data['comment']) === 0) {
            $error['Downtime']['comment'][] = __('Comment can not be empty');
        }

        if (!isset($data['from_date']) || strlen($data['from_date']) === 0) {
            $error['Downtime']['from_date'][] = __('Start date can not be empty');
        }

        if (!isset($data['from_time']) || strlen($data['from_time']) === 0) {
            $error['Downtime']['from_time'][] = __('Start time can not be empty');
        }

        if (!isset($data['to_date']) || strlen($data['to_date']) === 0) {
            $error['Downtime']['to_date'][] = __('End date can not be empty');
        }

        if (!isset($data['to_time']) || strlen($data['to_time']) === 0) {
            $error['Downtime']['to_time'][] = __('End time can not be empty');
        }

        if (empty($error['Downtime'])) {
            $start = sprintf('%s %s', $data['from_date'], $data['from_time']);
            $end = sprintf('%s %s', $data['to_date'], $data['to_time']);
            if (strtotime($start) === false) {
                $error['Downtime']['from_date'][] = __('Date is not valid');
            }

            if (strtotime($end) === false) {
                $error['Downtime']['to_date'][] = __('Date is not valid');
            }
        }

        if (!empty($error['Downtime'])) {
            $this->response = $this->response->withStatus(400);
            $this->set('success', false);
            $this->set('error', $error);
            $this->viewBuilder()->setOption('serialize', ['error', 'success']);
            return;

        }
        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();
        $offset = $UserTime->getUserTimeToServerOffset();

        $start = strtotime($start);
        $end = strtotime($end);

        $start = date('d.m.Y H:i', $start - $offset);
        $end = date('d.m.Y H:i', $end - $offset);

        $this->set('success', true);
        $this->set('error', $error);
        $this->set('start', $start);
        $this->set('end', $end);
        $this->viewBuilder()->setOption('serialize', ['error', 'success', 'start', 'end']);
    }

    public function icon() {
        //Only ship template
        return;
    }
}
