<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;

use App\Lib\SystemConfig;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Log\Log;
use Exception;
use GuzzleHttp\Exception\GuzzleException;
use itnovum\openITCOCKPIT\Perfdata\PerfdataLoader;


/**
 * Class GraphgeneratorsController
 * @package App\Controller
 */
class GraphgeneratorsController extends AppController {

    /**
     * New method for AngularJS
     * @throws GuzzleException
     */
    public function getPerfdataByUuid() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $hostUuid = $this->request->getQuery('host_uuid', '');
        $serviceUuid = $this->request->getQuery('service_uuid', '');
        $hours = $this->request->getQuery('hours', '');
        $start = (int)$this->request->getQuery('start', '');
        $end = (int)$this->request->getQuery('end', '');
        $jsTimestamp = (bool)$this->request->getQuery('jsTimestamp', 0);
        $gauge = $this->request->getQuery('gauge', '');
        $scale = $this->request->getQuery('scale', 'true') === 'true';
        $forcedUnit = $this->request->getQuery('forcedUnit', null);
        $aggregation = $this->request->getQuery('aggregation', 'avg');
        $debug = $this->request->getQuery('debug', 'false') === 'true';

        $SystemConfig = new SystemConfig();
        $PerfdataBackend = $SystemConfig->getPerfdataBackend();

        $PerfdataLoader = new PerfdataLoader($PerfdataBackend);
        if (is_numeric($hours)) {
            $hours = (int)$hours;
            $start = time() - ($hours * 3600);
            $end = time();
        }

        if ($start === null || $end === null) {
            $start = time() - (3 * 3600);
            $end = time();
        }

        try {
            $performance_data = $PerfdataLoader->getPerfdataByUuid($hostUuid, $serviceUuid, $start, $end, $jsTimestamp, $aggregation, $gauge, $scale, $forcedUnit, $debug);
            $this->set('performance_data', $performance_data);
            $this->viewBuilder()->setOption('serialize', ['performance_data']);

        } catch (Exception $e) {
            Log::error($e->getMessage());
            $performance_data[] = [
                'datasource' => [],
                'data'       => []
            ];
        }


        $this->set('performance_data', $performance_data);
        $this->viewBuilder()->setOption('serialize', ['performance_data']);
    }
}
