<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace App\Lib;


abstract class PluginAclDependencies {

    /**
     * Hold a list of controllers and actions which
     * should always be allowed and could not be disabled by the user
     *
     * For Example:
     * PagesController::paginator()
     * PagesController::csrf()
     * Users::login()
     * Users::logout()
     *
     * @var array
     */
    protected $allow = [];

    /**
     * Controller actions that depends on other controller actions.
     * This often happens when using a lot of Ajax in your frontend.
     *
     * For example:
     * Users::edit() make an Ajax call to Users::loadUsergroups() etc...
     *
     * @var array
     */
    protected $dependencies = [];

    public function __construct() {
        // Add actions that should always be allowed.
        //$this
        //    ->allow('Angular', 'index')
        //    ->allow('Angular', 'paginator');


        ///////////////////////////////
        //    Add dependencies       //
        //////////////////////////////
        //$this
        //    ->dependency('Agentchecks', 'add', 'Agentchecks', 'loadServicetemplates')
        //    ->dependency('Agentchecks', 'edit', 'Agentchecks', 'loadServicetemplates');
    }

    /**
     * @param string $controller
     * @param string $action
     * @return $this
     */
    protected function allow(string $controller, string $action): self {
        if (!isset($this->allow[$controller])) {
            $this->allow[$controller] = [];
        }

        $this->allow[$controller][$action] = $action;
        return $this;
    }

    /**
     * @param string $controller
     * @param string $action
     * @param string $dependentController
     * @param $dependentAction
     * @return $this
     */
    protected function dependency(string $controller, string $action, string $dependentController, $dependentAction): self {
        if (!isset($this->dependencies[$controller][$action])) {
            $this->dependencies[$controller][$action] = [];
        }

        if (!isset($this->dependencies[$controller][$action][$dependentController])) {
            $this->dependencies[$controller][$action][$dependentController] = [];
        }

        $this->dependencies[$controller][$action][$dependentController][] = $dependentAction;


        return $this;
    }

    /**
     * @return array
     */
    public function getAllow() {
        return $this->allow;
    }

    /**
     * @return array
     */
    public function getDependencies() {
        return $this->dependencies;
    }

}
