<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Lib;


use App\Application;
use Cake\ORM\TableRegistry;
use Cake\Datasource\FactoryLocator;
use Cake\Utility\Hash;
use Composer\Autoload\ClassLoader;
use itnovum\openITCOCKPIT\Core\FileDebugger;
use Symfony\Component\Finder\Finder;
use Symfony\Component\Finder\SplFileInfo;

class PluginManager {

    public static $associations = [];
    /**
     * @var array
     */
    private $modules = [];
    /**
     * @var Application
     */
    private $application;
    /**
     * @var bool
     */
    private $bootstrapPlugins;

    /**
     * PluginManager constructor.
     * @param Application $application
     * @param bool $bootstrapPlugins
     */
    public function __construct(Application $application, $bootstrapPlugins = true) {
        $this->application = $application;
        $this->bootstrapPlugins = $bootstrapPlugins;

        FactoryLocator::add('Table', new PluginManagerTableLocator()); // CakePHP >= 4.1
        //TableRegistry::setTableLocator(new PluginManagerTableLocator()); // CakePHP <= 4.1

        $this->_getAvailablePlugins();
        $this->addAllPlugins();
    }

    /**
     * @return array
     */
    private function _getAvailablePlugins() {
        $this->modules = self::getAvailablePlugins();
        return $this->modules;
    }

    public static function getAvailablePlugins() {
        $Finder = new Finder();
        $Finder->in(PLUGIN)
            ->directories()
            ->ignoreDotFiles(true)
            ->depth(0);

        $modules = [];

        /** @var SplFileInfo $folder */
        foreach ($Finder as $folder) {
            $modules[] = $folder->getFilename();
        }

        return $modules;
    }

    private function addAllPlugins() {
        /** @var ClassLoader $loader */
        $loader = require ROOT . '/vendor/autoload.php';

        //Autoload Plugin Classes
        foreach ($this->modules as $moduleName) {
            $loader->setPsr4(
                sprintf('%s\\', $moduleName),
                PLUGIN . $moduleName . DS . 'src'
            );
            $this->application->addPlugin($moduleName);


            $pluginAssociationsFile = PLUGIN . $moduleName . DS . 'config' . DS . 'associations.php';
            if (file_exists($pluginAssociationsFile)) {
                $mapping = require_once $pluginAssociationsFile;
                self::$associations = Hash::merge(self::$associations, $mapping);
            }

        }

        if ($this->bootstrapPlugins === true) {
            $this->application->pluginBootstrap();
        }
    }

}
