<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace App\Model\Entity;

use Cake\I18n\FrozenTime;
use Cake\ORM\Entity;
use itnovum\openITCOCKPIT\Core\Views\UserTime;


/**
 * DowntimeHost Entity
 *
 * @property string $hostname
 * @property int $internal_downtime_id
 * @property int $scheduled_start_time
 * @property string $node_name
 * @property int $entry_time
 * @property string|null $author_name
 * @property string|null $comment_data
 * @property int|null $triggered_by_id
 * @property bool|null $is_fixed
 * @property int|null $duration
 * @property int $scheduled_end_time
 * @property bool|null $was_started
 * @property int $actual_start_time
 * @property int $actual_end_time
 * @property bool|null $was_cancelled
 *
 */
class DowntimeHost extends Entity {

    /**
     * @var UserTime|null
     */
    private $UserTime = null;

    /**
     * Fields that can be mass assigned using newEntity() or patchEntity().
     *
     * Note that when '*' is set to true, this allows all unspecified fields to
     * be mass assigned. For security purposes, it is advised to set '*' to false
     * (or remove it), and explicitly make individual fields accessible as needed.
     *
     * @var array
     */
    protected $_accessible = [
        'entry_time'           => true,
        'author_name'          => true,
        'comment_data'         => true,
        'triggered_by_id'      => true,
        'is_fixed'             => true,
        'duration'             => true,
        'scheduled_start_time' => true,
        'scheduled_end_time'   => true,
        'was_started'          => true,
        'actual_start_time'    => true,
        'actual_end_time'      => true,
        'was_cancelled'        => true,
        'internal_downtime_id' => true,
    ];

    /**
     * @param UserTime $UserTime
     */
    public function setUserTime(UserTime $UserTime) {
        $this->UserTime = $UserTime;
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function jsonSerialize(): array {
        if ($this->UserTime === null && $this->BBCodeParser === null) {
            return parent::jsonSerialize();
        }

        $data = parent::jsonSerialize();
        if ($this->UserTime) {
            $fields = [
                'entry_time',
                'scheduled_start_time',
                'scheduled_end_time',
                'actual_start_time',
                'actual_end_time'
            ];

            foreach ($fields as $field) {
                if (isset($data[$field])) {
                    $data[$field] = $this->UserTime->format($data[$field]);
                }
            }

            $humanFields = [
                'duration'
            ];

            foreach ($humanFields as $field) {
                if (isset($data[$field])) {
                    $data[$field . '_for_humans'] = $this->UserTime->secondsInHumanShort($this->{$field});
                }
            }
        }

        if (isset($data['scheduled_end_time']) && isset($data['was_cancelled'])) {
            $data['is_cancellable'] = $this->scheduled_end_time > time() && $data['was_cancelled'] === false;
            $data['is_running'] = $data['was_started'] === true && $data['is_cancellable'];
            $data['is_expired'] = $data['was_started'] && $this->scheduled_end_time < time() && $this->was_cancelled === false;
        }

        return $data;
    }

}
