<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Model\Table;

use App\Lib\Traits\PaginationAndScrollIndexTrait;
use App\Model\Entity\AcknowledgementHost;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use itnovum\openITCOCKPIT\Core\AcknowledgedHostConditions;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * AcknowledgementHostsTable Model
 *
 * @method \Model\Entity\AcknowledgementHost newEmptyEntity()
 * @method \Model\Entity\AcknowledgementHost newEntity(array $data, array $options = [])
 * @method \Model\Entity\AcknowledgementHost[] newEntities(array $data, array $options = [])
 * @method \Model\Entity\AcknowledgementHost get($primaryKey, $options = [])
 * @method \Model\Entity\AcknowledgementHost findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \Model\Entity\AcknowledgementHost patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \Model\Entity\AcknowledgementHost[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \Model\Entity\AcknowledgementHost|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \Model\Entity\AcknowledgementHost saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \Model\Entity\AcknowledgementHost[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \Model\Entity\AcknowledgementHost[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \Model\Entity\AcknowledgementHost[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \Model\Entity\AcknowledgementHost[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class AcknowledgementHostsTable extends Table {

    use PaginationAndScrollIndexTrait;

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('statusengine_host_acknowledgements');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        //Readonly table
        return $validator;
    }

    /**
     * @param string $uuid
     * @return AcknowledgementHost|null
     */
    public function byUuid($uuid) {
        return $this->byHostUuid($uuid);
    }

    /**
     * @param null $uuid
     * @return array|AcknowledgementHost|null
     */
    public function byHostUuid($uuid = null) {
        $query = $this->find()
            ->where([
                'hostname' => $uuid
            ])
            ->order([
                'entry_time' => 'DESC',
            ])
            ->first();

        return $query;
    }

    /**
     * @param AcknowledgedHostConditions $AcknowledgedHostConditions
     * @param PaginateOMat|null $PaginateOMat
     * @return array
     */
    public function getAcknowledgements(AcknowledgedHostConditions $AcknowledgedHostConditions, $PaginateOMat = null) {
        $query = $this->find()
            ->where([
                'entry_time >' => $AcknowledgedHostConditions->getFrom(),
                'entry_time <' => $AcknowledgedHostConditions->getTo()
            ])
            ->order($AcknowledgedHostConditions->getOrder());

        if ($AcknowledgedHostConditions->hasConditions()) {
            $query->andWhere($AcknowledgedHostConditions->getConditions());
        }

        if (!empty($AcknowledgedHostConditions->getStates())) {
            $query->andWhere([
                'state IN' => $AcknowledgedHostConditions->getStates()
            ]);
        }


        if ($PaginateOMat === null) {
            //Just execute query
            $result = $this->emptyArrayIfNull($query->toArray());
        } else {
            if ($PaginateOMat->useScroll()) {
                $result = $this->scrollCake4($query, $PaginateOMat->getHandler());
            } else {
                $result = $this->paginateCake4($query, $PaginateOMat->getHandler());
            }
        }

        return $result;
    }
}
