<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Model\Table;

use App\Lib\Traits\PaginationAndScrollIndexTrait;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use itnovum\openITCOCKPIT\Core\AcknowledgedHostConditions;
use itnovum\openITCOCKPIT\Core\AcknowledgedServiceConditions;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * AcknowledgementServicesTable Model
 *
 * @method \Model\Entity\AcknowledgementService newEmptyEntity()
 * @method \Model\Entity\AcknowledgementService newEntity(array $data, array $options = [])
 * @method \Model\Entity\AcknowledgementService[] newEntities(array $data, array $options = [])
 * @method \Model\Entity\AcknowledgementService get($primaryKey, $options = [])
 * @method \Model\Entity\AcknowledgementService findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \Model\Entity\AcknowledgementService patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \Model\Entity\AcknowledgementService[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \Model\Entity\AcknowledgementService|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \Model\Entity\AcknowledgementService saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \Model\Entity\AcknowledgementService[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \Model\Entity\AcknowledgementService[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \Model\Entity\AcknowledgementService[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \Model\Entity\AcknowledgementService[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class AcknowledgementServicesTable extends Table {

    use PaginationAndScrollIndexTrait;

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('statusengine_service_acknowledgements');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        //Readonly table
        return $validator;
    }

    /**
     * @param string $uuid
     * @return AcknowledgementService|null
     */
    public function byUuid($uuid) {
        return $this->byServiceUuid($uuid);
    }

    /**
     * @param null $uuid
     * @return AcknowledgementService|null
     */
    public function byServiceUuid($uuid = null) {
        $query = $this->find()
            ->where([
                'service_description' => $uuid
            ])
            ->order([
                'entry_time' => 'DESC',
            ])
            ->first();

        return $query;
    }

    /**
     * @param AcknowledgedServiceConditions $AcknowledgedServiceConditions
     * @param PaginateOMat|null $PaginateOMat
     * @return array
     */
    public function getAcknowledgements(AcknowledgedServiceConditions $AcknowledgedServiceConditions, $PaginateOMat = null) {
        $query = $this->find()
            ->where([
                'entry_time >' => $AcknowledgedServiceConditions->getFrom(),
                'entry_time <' => $AcknowledgedServiceConditions->getTo()
            ])
            ->order($AcknowledgedServiceConditions->getOrder());

        if ($AcknowledgedServiceConditions->hasConditions()) {
            $query->andWhere($AcknowledgedServiceConditions->getConditions());
        }

        if (!empty($AcknowledgedServiceConditions->getStates())) {
            $query->andWhere([
                'state IN' => $AcknowledgedServiceConditions->getStates()
            ]);
        }


        if ($PaginateOMat === null) {
            //Just execute query
            $result = $this->emptyArrayIfNull($query->toArray());
        } else {
            if ($PaginateOMat->useScroll()) {
                $result = $this->scrollCake4($query, $PaginateOMat->getHandler());
            } else {
                $result = $this->paginateCake4($query, $PaginateOMat->getHandler());
            }
        }

        return $result;
    }
}
