<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\itnovum\openITCOCKPIT\Database;

class SanitizeOrder {

    /**
     * Only allow to pass column_name or Table.column_name into the MySQL ORDER BY query
     *
     * @param string $sort
     * @return array|string|string[]|null
     */
    public static function filterOrderColumn(string $sort) {
        $sort = trim($sort);
        // Only match to column_name or Table.column_name (no numbers, no special characters or umlauts)
        if (preg_match('/^[a-zA-Z_]+(\.[a-zA-Z_]+)*$/', $sort) === 1) {
            // This should be unnecessary because the preg_match above should never match on invalid characters
            // However, better safe than sorry
            return preg_replace('/[^a-zA-Z_\.]/', '', $sort);
        }
        // Ref.: https://xkcd.com/327/
        throw new FishyQueryException('Bobby Tables?');
    }

    public static function filterOrderArray($sortAsArray) {
        if (is_array($sortAsArray)) {
            $validatedSort = [];
            foreach ($sortAsArray as $sortField => $sortDirection) {
                $sortField = self::filterOrderColumn($sortField);
                $validatedSort[$sortField] = ($sortDirection === 'desc') ? 'desc' : 'asc';
            }
            return $validatedSort;
        }

        return self::filterOrderColumn((string)$sortAsArray);
    }
}
