<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\itnovum\openITCOCKPIT\Monitoring\Naemon;

use App\Model\Table\ServicestatusTable;
use Cake\ORM\TableRegistry;

class ExternalCommands {

    /**
     * @var array
     */
    private $gearmanConfig = [
        'address' => '127.0.0.1',
        'port'    => 4730
    ];

    public function __construct() {
    }

    /**
     * @param string $hostUuid
     * @param bool $includeServices
     * @param int|null $timestamp
     */
    public function rescheduleHost(string $hostUuid, bool $includeServices = false, ?int $timestamp = null) {
        if ($timestamp === null) {
            $timestamp = time();
        }

        if ($includeServices === false) {
            //SCHEDULE_FORCED_HOST_CHECK
            $payload = [
                'Command' => 'schedule_check',
                'Data'    => [
                    'host_name'     => $hostUuid,
                    'schedule_time' => $timestamp
                ]
            ];
            $this->toQueue($payload);
        } else {
            //SCHEDULE_FORCED_HOST_CHECK
            $payload = [
                'Command' => 'schedule_check',
                'Data'    => [
                    'host_name'     => $hostUuid,
                    'schedule_time' => $timestamp
                ]
            ];
            $this->toQueue($payload);

            //SCHEDULE_FORCED_HOST_SVC_CHECKS
            $payload = [
                'Command' => 'raw',
                'Data'    => sprintf('SCHEDULE_FORCED_HOST_SVC_CHECKS;%s;%s', $hostUuid, $timestamp)
            ];
            $this->toQueue($payload);
        }
    }

    /**
     * @param string $hostUuid
     * @param string $serviceUuid
     * @param int|null $timestamp
     */
    public function rescheduleService(string $hostUuid, string $serviceUuid, ?int $timestamp = null) {
        if ($timestamp === null) {
            $timestamp = time();
        }

        //SCHEDULE_FORCED_SVC_CHECK
        $payload = [
            'Command' => 'schedule_check',
            'Data'    => [
                'host_name'           => $hostUuid,
                'service_description' => $serviceUuid,
                'schedule_time'       => $timestamp,
            ]
        ];
        $this->toQueue($payload);
    }

    /**
     * @param string $hostUuid
     * @param int $startTime
     * @param int $endTime
     * @param string $author
     * @param string $comment
     * @param bool $includeServices
     * @param int|null $duration
     */
    public function setHostDowntime(string $hostUuid, int $startTime, int $endTime, string $author, string $comment, bool $includeServices = false, ?int $duration = null) {
        if ($duration === null) {
            $duration = $endTime - $startTime;
        }

        //SCHEDULE_HOST_DOWNTIME
        $payload = [
            'Command' => 'raw',
            'Data'    => sprintf(
                'SCHEDULE_HOST_DOWNTIME;%s;%s;%s;1;0;%s;%s;%s',
                $hostUuid,
                $startTime,
                $endTime,
                $duration,
                $author,
                $comment
            )
        ];

        //Set Host downtime on the Satellite System
        $this->toQueue($payload);

        //Also set Host downtime on the Master System
        $this->toMasterQueue($payload);

        if ($includeServices) {
            //SCHEDULE_HOST_SVC_DOWNTIME
            $payload = [
                'Command' => 'raw',
                'Data'    => sprintf(
                    'SCHEDULE_HOST_SVC_DOWNTIME;%s;%s;%s;1;0;%s;%s;%s',
                    $hostUuid,
                    $startTime,
                    $endTime,
                    $duration,
                    $author,
                    $comment
                )
            ];
            //Set Host+Services downtime on the Satellite System
            $this->toQueue($payload);

            //Also set Host+Services downtime on the Master system
            $this->toMasterQueue($payload);
        }
    }

    /**
     * @param string $hostUuid
     * @param string $serviceUuid
     * @param int $startTime
     * @param int $endTime
     * @param string $author
     * @param string $comment
     * @param int|null $duration
     */
    public function setServiceDowntime(string $hostUuid, string $serviceUuid, int $startTime, int $endTime, string $author, string $comment, ?int $duration = null) {
        if ($duration === null) {
            $duration = $endTime - $startTime;
        }

        //SCHEDULE_SVC_DOWNTIME
        $payload = [
            'Command' => 'raw',
            'Data'    => sprintf(
                'SCHEDULE_SVC_DOWNTIME;%s;%s;%s;%s;1;0;%s;%s;%s',
                $hostUuid,
                $serviceUuid,
                $startTime,
                $endTime,
                $duration,
                $author,
                $comment
            )
        ];

        //Set Service downtime on the Satellite System
        $this->toQueue($payload);

        //Also set Service downtime on the Master System
        $this->toMasterQueue($payload);
    }

    /**
     * @param string $hostUuid
     * @param string $author
     * @param string $comment
     * @param bool $isSticky
     * @param bool $includeServices
     */
    public function setHostAck(string $hostUuid, string $author, string $comment, bool $isSticky = false, bool $includeServices = false) {
        // ACKNOWLEDGE_HOST_PROBLEM;<host_name>;<sticky>;<notify>;<persistent>;<author>;<comment>
        $payload = [
            'Command' => 'raw',
            'Data'    => sprintf(
                'ACKNOWLEDGE_HOST_PROBLEM;%s;%d;1;1;%s;%s',
                $hostUuid,
                $isSticky ? 2 : 0, // 2 = sticky with Nagios Core, 1 = Sticky with Naemon Core, 2 works for both :)
                $author,
                $comment
            )
        ];

        //Set Host Ack on the Satellite System
        $this->toQueue($payload);

        //Also set Host Ack on the Master System
        $this->toMasterQueue($payload);

        if ($includeServices === true) {
            /** @var ServicestatusTable $ServicestatusTable */
            $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

            foreach ($ServicestatusTable->getServicesByHostUuidForAck($hostUuid) as $service) {
                $this->setServiceAck(
                    $hostUuid,
                    $service['service_description'],
                    $author,
                    $comment,
                    $isSticky
                );
            }
        }
    }

    /**
     * @param string $hostUuid
     * @param string $serviceUuid
     * @param string $author
     * @param string $comment
     * @param bool $isSticky
     */
    public function setServiceAck(string $hostUuid, string $serviceUuid, string $author, string $comment, bool $isSticky = false) {
        // ACKNOWLEDGE_SVC_PROBLEM;<host_name>;<service_description>;<sticky>;<notify>;<persistent>;<author>;<comment>
        $payload = [
            'Command' => 'raw',
            'Data'    => sprintf(
                'ACKNOWLEDGE_SVC_PROBLEM;%s;%s;%d;1;1;%s;%s',
                $hostUuid,
                $serviceUuid,
                $isSticky ? 2 : 0, // 2 = sticky with Nagios Core, 1 = Sticky with Naemon Core, 2 works for both :)
                $author,
                $comment
            )
        ];

        //Set Service Ack on the Satellite System
        $this->toQueue($payload);

        //Also set Service Ack on the Master System
        $this->toMasterQueue($payload);
    }

    /**
     * @param string $hostUuid
     * @param string $output
     * @param int $state
     * @param int $repetitions
     * @param string $longOutput
     */
    public function passiveTransferHostCheckresult(string $hostUuid, string $output = 'No output given', int $state = 2, int $repetitions = 1, string $longOutput = '') {
        for ($i = 1; $i <= $repetitions; $i++) {
            //PROCESS_HOST_CHECK_RESULT
            $payload = [
                'Command' => 'check_result',
                'Data'    => [
                    'host_name'     => $hostUuid,
                    'output'        => $output,
                    'long_output'   => $longOutput,
                    'check_type'    => 1, //https://github.com/naemon/naemon-core/blob/cec6e10cbee9478de04b4cf5af29e83d47b5cfd9/src/naemon/common.h#L330-L334
                    'return_code'   => $state,
                    'start_time'    => time(),
                    'end_time'      => time(),
                    'early_timeout' => 0,
                    'latency'       => 0,
                    'exited_ok'     => 1
                ]
            ];

            //Set passive host check result on Satellite System
            $this->toQueue($payload);
        }
    }

    /**
     * @param string $hostUuid
     * @param string $serviceUuid
     * @param string $output
     * @param int $state
     * @param int $repetitions
     * @param string $longOutput
     */
    public function passiveTransferServiceCheckresult(string $hostUuid, string $serviceUuid, string $output = 'No output given', int $state = 3, int $repetitions = 1, string $longOutput = '') {
        for ($i = 1; $i <= $repetitions; $i++) {
            //PROCESS_SERVICE_CHECK_RESULT
            $payload = [
                'Command' => 'check_result',
                'Data'    => [
                    'host_name'           => $hostUuid,
                    'service_description' => $serviceUuid,
                    'output'              => $output,
                    'long_output'         => $longOutput,
                    'check_type'          => 1, //https://github.com/naemon/naemon-core/blob/cec6e10cbee9478de04b4cf5af29e83d47b5cfd9/src/naemon/common.h#L330-L334
                    'return_code'         => $state,
                    'start_time'          => time(),
                    'end_time'            => time(),
                    'early_timeout'       => 0,
                    'latency'             => 0,
                    'exited_ok'           => 1
                ]
            ];

            //Set passive service check result on Satellite System
            $this->toQueue($payload);
        }
    }

    /**
     * @param string $hostUuid
     * @param array $downtime
     */
    public function deleteHostDowntime(string $hostUuid, array $downtime) {
        //DEL_HOST_DOWNTIME
        $payload = [
            'Command' => 'delete_downtime',
            'Data'    => [
                'host_name'  => $hostUuid,
                'start_time' => $downtime['scheduled_start_time'],
                'end_time'   => $downtime['scheduled_end_time'],
                'comment'    => $downtime['comment_data']
            ]
        ];

        // Delete downtime on the Satellite System
        $this->toQueue($payload);

        //Also delete Downtime on the Master System
        $this->toMasterQueue($payload);
    }

    /**
     * @param string $hostUuid
     * @param string $serviceUuid
     * @param array $downtime
     */
    public function deleteServiceDowntime(string $hostUuid, string $serviceUuid, array $downtime) {
        //DEL_SVC_DOWNTIME
        $payload = [
            'Command' => 'delete_downtime',
            'Data'    => [
                'host_name'           => $hostUuid,
                'service_description' => $serviceUuid,
                'start_time'          => $downtime['scheduled_start_time'],
                'end_time'            => $downtime['scheduled_end_time'],
                'comment'             => $downtime['comment_data']
            ]
        ];

        // Delete downtime on the Satellite System
        $this->toQueue($payload);

        //Also delete Downtime on the Master System
        $this->toMasterQueue($payload);

    }

    public function enableOrDisableHostFlapdetection(string $hostUuid, bool $enable = true) {
        if ($enable === true) {
            $payload = [
                'Command' => 'raw',
                'Data'    => sprintf('ENABLE_HOST_FLAP_DETECTION;%s', $hostUuid)
            ];

            //Set enable host flap detection on the Satellite System
            $this->toQueue($payload);

            //Also set enable host flap detection on the Master System
            $this->toMasterQueue($payload);
        } else {
            $payload = [
                'Command' => 'raw',
                'Data'    => sprintf('DISABLE_HOST_FLAP_DETECTION;%s', $hostUuid)
            ];

            //Set disable host flap detection on the Satellite System
            $this->toQueue($payload);

            //Also set disable host flap detection on the Master System
            $this->toMasterQueue($payload);
        }
    }

    public function enableOrDisableServiceFlapdetection(string $hostUuid, string $serviceUuid, bool $enable = true) {
        if ($enable === true) {
            $payload = [
                'Command' => 'raw',
                'Data'    => sprintf('ENABLE_SVC_FLAP_DETECTION;%s;%s', $hostUuid, $serviceUuid)
            ];

            //Set enable service flap detection on the Satellite System
            $this->toQueue($payload);

            //Also set enable service flap detection on the Master System
            $this->toMasterQueue($payload);
        } else {
            $payload = [
                'Command' => 'raw',
                'Data'    => sprintf('DISABLE_SVC_FLAP_DETECTION;%s;%s', $hostUuid, $serviceUuid)
            ];

            //Set disable service flap detection on the Satellite System
            $this->toQueue($payload);

            //Also set disable service flap detection on the Master System
            $this->toMasterQueue($payload);
        }
    }

    /**
     * @param array $payload
     */
    public function toQueue(array $payload) {
        $GearmanClient = new \GearmanClient();
        $GearmanClient->addServer($this->gearmanConfig['address'], $this->gearmanConfig['port']);
        $GearmanClient->setTimeout(5000);

        if ($payload['Command'] === 'raw') {
            // This is like passing an external command to nagios.cmd or naemon.cmd
            // Its the exact same syntax so we need to prefix the data with the current timestamp

            $payload['Data'] = sprintf('[%s] %s', time(), $payload['Data']);
        }

        // Command is for the local running Naemon
        $GearmanClient->doBackground('statusngin_cmd', json_encode($payload));
    }

    /**
     * @param array $payload
     */
    public function toMasterQueue(array $payload) {
        $GearmanClient = new \GearmanClient();
        $GearmanClient->addServer($this->gearmanConfig['address'], $this->gearmanConfig['port']);
        $GearmanClient->setTimeout(5000);

        if ($payload['Command'] === 'raw') {
            // This is like passing an external command to nagios.cmd or naemon.cmd
            // Its the exact same syntax so we need to prefix the data with the current timestamp

            $payload['Data'] = sprintf('[%s] %s', time(), $payload['Data']);
        }

        // Command is for the Naemon running on the openITCOCKPIT Master Server
        $GearmanClient->doBackground('statusngin_cmd_mastertx', json_encode($payload));
    }

    public function toStatusnginCmdBackground(array $payload) {
        $GearmanClient = new \GearmanClient();
        $GearmanClient->addServer($this->gearmanConfig['address'], $this->gearmanConfig['port']);
        $GearmanClient->setTimeout(5000);

        return $GearmanClient->doBackground('statusngin_cmd', json_encode($payload));
    }

}
