<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class Battery
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class Battery extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('sensors')) {
            return;
        }
        if ($this->checkError('sensors', 'Batteries')) {
            return;
        }

        $requiredKeys = [
            'sensors' => [
                'Batteries'
            ]
        ];

        foreach ($requiredKeys as $mainKey => $subKeys) {
            foreach ($subKeys as $subKey) {
                if (!isset($this->json[$mainKey][$subKey])) {
                    $this->setOutput(sprintf('Key "%s.%s" not found in Agent output', $mainKey, $subKey));
                    $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                    return;
                }
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $warning = '@35';
        $critical = '@20';
        $id = 0;
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $warning = $arg;
                    break;
                case 1:
                    $critical = $arg;
                    break;
                case 2:
                    $id = (int)$arg;
                    break;
            }
        }

        foreach ($this->json['sensors']['Batteries'] as $battery) {
            if (isset($battery['id'])) {
                if ($battery['id'] === $id) {
                    $this->addPerfdata(new PerfdataGauge(
                        'percent',
                        round($battery['percent'], 2),
                        '%',
                        $warning,
                        $critical,
                        0,
                        100
                    ));


                    // Not implemented yet in gopsutil
                    //$this->addPerfdata(new PerfdataGauge(
                    //    'secsleft',
                    //    $battery['secsleft'],
                    //    's'
                    //));

                    $this->setStatuscode(NagiosService::STATUS_OK);
                    $this->setOutput(sprintf(
                        'Battery loaded: %s%%, Power plugged: %s',
                        round($battery['percent'], 2),
                        $battery['power_plugged'] ? 'Yes' : 'No'
                    ));

                    if ($this->getRangeStatus($critical, $battery['percent'])) {
                        $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                        return;
                    }

                    if ($this->getRangeStatus($warning, $battery['percent'])) {
                        $this->setStatuscode(NagiosService::STATUS_WARNING);
                        return;
                    }
                    return;
                }
            }
        }

        $this->setOutput(sprintf('Battery with id "%s" not found!', $id));
        $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
    }
}
