<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects;

class TargetHost {

    /**
     * The JSON config as array
     * @var array
     */
    private $config = [];

    /**
     * @var float
     */
    private $timeout;

    /**
     * TargetHost constructor.
     * @param array $config
     * @param float $timeout
     */
    public function __construct($config = [], $timeout = 30.0) {
        $this->config = $config;
        $this->timeout = $timeout;
    }

    /**
     * @return string
     */
    public function getAddress() {
        return $this->config['address'];
    }

    /**
     * @return int
     */
    public function getPort() {
        return $this->config['port'];
    }

    /**
     * @return string
     */
    public function getHostUuid() {
        return $this->config['uuid'];
    }

    /**
     * @return string
     */
    public function getHostname() {
        return $this->config['name'];
    }

    /**
     * @return float
     */
    public function getTimeout() {
        return $this->timeout;
    }

    /**
     * @return bool
     */
    public function useAutoTLS() {
        return $this->config['use_autossl'] === true;
    }

    /**
     * @return bool
     */
    public function useHttps() {
        return $this->config['use_https'] === true;
    }

    /**
     * @return bool
     */
    public function allowInsecureHttps() {
        return $this->config['insecure'] === true;
    }

    /**
     * @return bool
     */
    public function useHttp() {
        return $this->config['use_autossl'] === false && $this->config['use_https'] === false;
    }

    /**
     * @return bool
     */
    public function useBasicAuth() {
        return $this->config['basic_auth'] === true;
    }

    /**
     * @return string
     */
    public function getUsername() {
        return $this->config['username'];
    }

    /**
     * @return string
     */
    public function getPassword() {
        return $this->config['password'];
    }

    /**
     * @return bool
     */
    public function isPullMode() {
        return $this->config['mode'] === 'pull';
    }

    /**
     * @return bool
     */
    public function hasChecks() {
        return !empty($this->config['checks']);
    }

    /**
     * @return array
     */
    public function getChecksConfig() {
        return $this->config['checks'];
    }

    /**
     * @return bool|string
     */
    public function getProxyAddress() {
        if ($this->config['proxy']) {
            return $this->config['proxy'];
        }

        return false;
    }

}
